SoundSettings = { --Edit these to chance the sounds in game.
	--Pitch should probably be kept within -1~1, but I'm not sure how the game handles them.
	colorpitch	= 1.00,
	greypitch	= 1.00,
}

GameplaySettings{
		usepuzzlegrid					= false,
		puzzlerows						= 7,
		puzzlecols						= 3,
        greypercent						= 0.4,
        colorcount						= 1,
        usetraffic						= true,
        automatic_traffic_collisions	= false, -- the game shouldn't check for block collisions since we'll be doing that ourselves in this script
        jumpmode						= "none",
	-- pointspergrey = -100,
        matchcollectionseconds			= 1.5,
        greyaction						= "normal", -- "eraseall"  -- "eraseblock"

		--track generation settings
		gravity						= -0.45, -- even without jumping the gravity setting is (a little bit) relevant. It's used in generating the track to sculpt it steep enough to allow jumps
        playerminspeed				= 0.1,--so the player is always moving somewhat
        playermaxspeed				= 2.5,--2.5
        minimumbestjumptime			= 2.5,--massage the track until a jump of at least this duration is possible
        uphilltiltscaler			= 1,--set to 1 for normal track. higher for steeper
        downhilltiltscaler			= 1,--set to 1 for normal track. higher for steeper
        uphilltiltsmoother			= 0.03,
        downhilltiltsmoother		= 0.06,
        useadvancedsteepalgorithm	= true, --set false for a less extreme track
        alldownhill					= false,
		--end track generation settings
}

SetSkinProperties{
	lanedividers={-1.5,1.5},
	shoulderlines={-4.5,4.5},
	trackwidth = 5,
	prefersteep = true
}

function deepcopy(orig) -- a pretty standard lua function for making a copy of a table
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

player={
	score		= 0,
	prevInput	= {},
	iPrevRing	= 0,
	hasFinishedScoringPrevRing = false,
	uniqueName	= "Player",
	num			= 1,
	prevFirstBlockCollisionTested = 1,
	pos			= {0,0,0},
	posCosmetic	= {0,0,1.5},
	controller	= "mouse",
	points		= 0 -- used for accumulating points this player earns at each match collection. temp var
}


function OnTrackCreated(theTrack)--track is created before the traffic
	track = theTrack --store a global copy of the track to maybe use later
	-- when you return a track table from this function the game will read and apply any changes you made
end

lanespace		= 3
half_lanespace	= 1.5

blocks			= blocks or {}
blockNodes		= blockNodes or {}
blockOffsets	= blockOffsets or {}
blockColors		= blockColors or {}

function OnTrafficCreated(theTraffic)
	half_lanespace = lanespace / 2

	traffic = theTraffic

	total_traffic = 0
	color_traffic = 0
    for i = 1, #traffic do
    	total_traffic = total_traffic + 1
    	local lane = traffic[i].lane -- math.random(-1,1)
--    	traffic[i].lane = lane
    	local strafe = lane*lanespace
--    	traffic[i].strafe = strafe
    	--local lane = traffic[i].lane
    	local offset = {strafe,0,0}
--    	for j=traffic[i].chainstart, traffic[i].chainend do
		local block = {}
		block.lane = lane
		block.hidden = false
		--block.tested = false
		block.tested = false
		--for j=1,#players do
		--	block.tested[j] = false --each player must collision test each block seperately since players can (sometimes) collide with the same blocks as each other
		--end
		block.type = traffic[i].type
		blocks[#blocks+1]=block
		--astroidLanes[#astroidLanes+1] = lane
		blockNodes[#blockNodes+1] = traffic[i].impactnode
		blockOffsets[#blockOffsets+1] = offset
		blockColors[#blockColors+1] = track[traffic[i].impactnode].color
		if blocks[i].type ~= 5 then
			color_traffic = color_traffic + 1
		end
--    	end
    end

    -- return traffic -- when you return a traffic table from this function the game will read and apply any changes you made
end

beakerPos = {0,0,1.5}
beakerScale = {.7,.7,.7}
function OnSkinLoaded()-- called after OnTrafficCreated. The skin script has loaded content.
	CreateClone{name="beaker", prefabName="Vehicle", attachToTrackWithNodeOffset=-1, transform={pos=beakerPos,scale=beakerScale}}
	SetScoreboardNote{text="STEALTH | Total color traffic: "..tostring(color_traffic)}
end

score = 0 --the global score (in multiplayer, shared by all players co-operatively)
function OnPuzzleCollecting()
	--Do nothing
end

iCurrentRing = 0 --Update function keeps this current
blocksToHide = {}
stealthy = true

function Collide(strafe, hitAllLanes)
	local playerLane = 0;
	if strafe>half_lanespace then playerLane = 1
	elseif strafe<-half_lanespace then playerLane=-1 end

	local maxRing = iCurrentRing + 2
	local foundFirst = false
	for i=player.prevFirstBlockCollisionTested,#blockNodes do
		if not blocks[i].tested then
			if blockNodes[i] <= maxRing then
				if not foundFirst then
					player.prevFirstBlockCollisionTested = i
					foundFirst = true
				end

				if blockNodes[i] <= math.floor(iCurrentRing) then
					if not blocks[i].hidden then
						if (blocks[i].lane == playerLane) or hitAllLanes then
							blocksToHide[#blocksToHide+1] = i
							blocks[i].hidden = true
							local blockOffset = blockOffsets[i]
							if blocks[i].type == 5 then
								score = score - 26;
								SetGlobalScore{score=score,showdelta=true}
								if stealthy then
									stealthy = false
									PlaySound{name="landcrash",pitch=SoundSettings.greypitch} -- play crash sound when they lose stealth
									SetScoreboardNote{text="Total color traffic: "..tostring(color_traffic)}
								end
							else
							score = score + 10;
							SetGlobalScore{score=score,showdelta=true}

							end
							SendCommand{command="HoverUp"} -- causes the hovering vehicle to bounce up a bit
						end
					end
					blocks[i].tested = true
				end
			else
				break --stop the loop once we get to a block way past the player
			end
			--end
		end
	end
end

function GetNextCollisionStrafe() -- figure out which lane the minion should be in
	for i=player.prevFirstBlockCollisionTested,#blockNodes do
		if not blocks[i].tested then
			return blocks[i].lane * lanespace
		end
	end

	return 0
end

prevHitAllLanes = true
function Update(dt, tracklocation, playerstrafe, input) --called every frame
	--iCurrentRing = math.floor(GetCurrentTrackLocation())
	--local input = GetInput()
	iCurrentRing = math.floor(tracklocation)
	local playersInput = input["players"]
	local mouseInput = input["mouse"]

	local hitAllLanes = false
	if stealthy then
		if playersInput["Button 1"] or mouseInput["LMB"] then
			hitAllLanes = true
		end
	end

	if prevHitAllLanes ~= hitAllLanes then
		if hitAllLanes then
			SendCommand{command="Show", name="beaker"}
			--SendCommand{command="Show", name="wingman2"}
			--SendCommand{command="Show", name="wingman3"}
		else
			SendCommand{command="Hide", name="beaker"}
			--SendCommand{command="Hide", name="wingman2"}
			--SendCommand{command="Hide", name="wingman3"}
		end

		prevHitAllLanes = hitAllLanes
	end

	blocksToHide = {}
	Collide(playerstrafe, hitAllLanes)

	if #blocksToHide > 0 then
		PlaySound{name="hit",pitch=SoundSettings.colorpitch}
		HideTraffic(blocksToHide)
	end

	--move the helper to line it up for the next block hit
	local desiredHelperStrafe = GetNextCollisionStrafe()
	beakerPos[1] = beakerPos[1] + dt * 11 * (desiredHelperStrafe - beakerPos[1])
	SendCommand{command="SetTransform", name="beaker", param={pos=beakerPos,scale=beakerScale}}
end

function OnRequestFinalScoring()
	local cleanFinishBonus = 0

	local stealthBonus = 0
	if stealthy then stealthBonus = score * .2 end

	return {
		rawscore = score,
		bonuses = {
			"Stealth Bonus:"..stealthBonus,
		},
		finalscore = score + stealthBonus
	}
end